#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <time.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <syslog.h>
#include "hcnetsdk.h"
#include <glib.h>

long UserID;
long lRealPlayHandle;

void log_message(char *msg);
void record(long lRealPlayHandle, char *folder, char *camera);
void daemon();
void stop();
void signal_handler(int sig);

void log_message(char *msg)
{
openlog("HIK Vision getVideo",0, LOG_USER);
syslog(LOG_NOTICE,msg);
closelog();
}

void daemon()
{
int i;
if (getppid() == 1) return;
i = fork();
if (i<0) exit(1); 
if (i>0) exit(0); 
setsid();
signal(SIGCHLD,SIG_IGN); 
signal(SIGTSTP,SIG_IGN); 
signal(SIGTTOU,SIG_IGN);
signal(SIGTTIN,SIG_IGN);
signal(SIGHUP,SIG_IGN); 
signal(SIGTERM,signal_handler);
signal(SIGKILL,signal_handler); 
}
void signal_handler(int sig){
    switch(sig) {
    case SIGTERM:
        stop();
        break;
    case SIGKILL:
	stop();
    }
}
void stop ()
{
log_message("Save data and exit...");
NET_DVR_StopRealPlay(lRealPlayHandle);
NET_DVR_Logout(UserID);
NET_DVR_Cleanup();
exit(0);
}

int main(int argc, char **argv)
{
GKeyFile *gkey = g_key_file_new();
GError *error;

char *ip,*login ,*pass;
char *folder,*camera;
FILE *IniFile;
int port=0;
struct hostent *he;
struct in_addr a;
int timeout=10;
int err;

if (argc < 2)
{
printf( "Usage: %s config.ini\n",argv[0]);
return 0;
}
if (! g_key_file_load_from_file(gkey,argv[1],G_KEY_FILE_KEEP_COMMENTS,&error))
{
	printf("Error open config gile %s\n",argv[1]);
	return 0;
}
daemon();
ip =  g_key_file_get_string(gkey,"Network","host",&error);
port =  atoi(g_key_file_get_string(gkey,"Network","port",&error));
login =  g_key_file_get_string(gkey,"Network","login",&error);
pass =  g_key_file_get_string(gkey,"Network","password",&error);
timeout = atoi(g_key_file_get_string(gkey,"Network","record_time",&error));
folder =  g_key_file_get_string(gkey,"SaveData","folder",&error);
camera =  g_key_file_get_string(gkey,"SaveData","camera",&error);

NET_DVR_Init();
NET_DVR_SetConnectTime(2000,1);
NET_DVR_SetReconnect(10000,1);

NET_DVR_DEVICEINFO struDeviceInfo;
he = gethostbyname (ip);
bcopy(*he->h_addr_list++, (char *) &a, sizeof(a));
while(1)
{
UserID = NET_DVR_Login(inet_ntoa(a),port,login,pass,&struDeviceInfo);
if (UserID < 0 )
{
	err = NET_DVR_GetLastError();
	switch(err)
	{
		case 1:
			log_message("Login/Password error\n");
			exit(0);
		case 7:
			log_message("Failed to connect to the device. The device is off-line, or connection timeout caused by network.\n");
			break;
	}
sleep(5);
}
else {
log_message("Login ok\n");
break;
}
}

NET_DVR_CLIENTINFO ClientInfo = {0};
ClientInfo.lChannel     = 1;
ClientInfo.lLinkMode    = 0;
ClientInfo.sMultiCastIP = NULL;
lRealPlayHandle = NET_DVR_RealPlay(UserID, &ClientInfo); 
 if (lRealPlayHandle < 0)
    {
        printf("NET_DVR_RealPlay error\n");
	printf("%d\n",NET_DVR_GetLastError());
        return 0;
    }

while(1)
{
record(lRealPlayHandle,folder,camera);
sleep(timeout);
NET_DVR_StopSaveRealData(lRealPlayHandle);
}
//NET_DVR_StopSaveRealData(lRealPlayHandle);
NET_DVR_StopRealPlay(lRealPlayHandle);
NET_DVR_Logout(UserID);
NET_DVR_Cleanup();

    return 0;
} 
void record(long lRealPlayHandle, char *folder, char *camera)
{
char filename[255];
time_t t;
struct tm *area;

t = time(NULL);
area = localtime(&t);
sprintf(filename,"%s/%s_%02d%02d%02d%02d%02d%02d.flv",folder,camera,
area->tm_year+1900,area->tm_mon+1,area->tm_mday,area->tm_hour,area->tm_min,area->tm_sec);
NET_DVR_SaveRealData(lRealPlayHandle,filename);
}
